/*
  Program:      Mine Mayhem
  File:         times.h
  Date: 	July, 1996 to February, 1997, version 1.00.
		26 October to 3 December,     version 2.00.

  Definitions for record times.

  Freeware, Copyright 1996-7 Jason Hood.

  You are free to use this code, or a portion thereof, as long as an
  appropriate acknowledgement is made.

  I was planning to use radio buttons to select the game and the level, but
  the radio button has no events associated with it, so I've had to use
  checkboxes instead.

  v2.00: added Table::str() and Table::Entry::str() functions;
	 added highlight parameter.
*/

#ifndef _times_h
#define _times_h

#include <tws/tabwin.h>
#include <tws/static.h>
#include <tws/edit.h>
//#include <tws/radio.h>
#include <tws/checkbox.h>
#include <tws/menu.h>
#include <tws/button.h>

#include "layout.h"
#include "options.h"
#include "custom.h"

extern FONT *FixedFont, *SysFont1;


const unsigned NameLen = 20 + 1,
               DateLen = 11 + 1,        // "dd-Mmm-yyyy\0"
               Entries = 5;


// This class holds the five fastest and slowest times.
class Table
{
friend class TimesWindow;

public:
  Table();

  int	Enter(unsigned time, unsigned spaces, unsigned mines);
					// Check if time (in ms) is a record
  char* str(bool clear) const;		// Format the statistics as a string

protected:
  unsigned games,                       // The number of games completed
           timetotal,                   // The total time for all games
           spacetotal,                  // Total number of spaces cleared
           minetotal;                   // Total number of mines found
					// (Above two for the cross game)

  struct Entry
  {
    Entry();
    char* str(bool clear, bool deviation, double avg) const;
					// Format the entry as a string

    unsigned time;                      // In milliseconds
    char     name[NameLen],
	     date[DateLen];
    int      start,			// Starting condition
	     finish;			// Finishing condition
    bool     wrap;			// Wrap-around (all three for clear)
    unsigned spaces,			// Spaces cleared
	     mines;			// Mines flagged (both for cross)

  } fastest[Entries], slowest[Entries];

  void shift(Entry which[], unsigned from);	// Make room for the new entry
  void GetDate(Entry& entry);
  void AskName(Entry which[], unsigned position);
  void Stats(Entry which[], unsigned position,
	     unsigned time, unsigned spaces, unsigned mines);
};


// This class holds the tables for each level and option.
class Times
{
friend class TimesWindow;

public:
  Times();
  ~Times();

  bool Check(int level, unsigned time, unsigned spaces, unsigned mines);
                                        // Is time a record for level?
  void Display(int level);              // Display the times for level

  bool Save(const char* filename);
  bool Load(const char* filename, CustomPopupRes& custom);

protected:
  Table* FindTable(int level, int game = Game, bool automark = isAutoMark,
                                               bool autoopen = isAutoOpen);

  Table Beginner[6], Intermediate[6], Expert[6];

  struct CustomTimes
  {
    Layout	 layout;
    Table	 times[6];
    CustomTimes* next;
  } *head, *prevCustom;
};


// This is the class that does all the displaying.
class TimesWindow : public TabWin
{
public:
  TimesWindow(const Table& table, int level, int highlight = -1);
  ~TimesWindow();

  void Run();

protected:
  void PaintWindow(int x1, int y1, int x2, int y2);

  void Print(const Table& table, int highlight = -1);

  void cmOK() { ws.StopRunningEvents(); }

  void CheckGameClear();
  void CheckGameCross();
  void UnCheckGameClear();
  void UnCheckGameCross();

  void CheckAutoMark();
  void CheckAutoOpen();

  void CheckLevelBeginner();
  void CheckLevelIntermediate();
  void CheckLevelExpert();
  void CheckLevelCustom();
  void UnCheckLevelBeginner();
  void UnCheckLevelIntermediate();
  void UnCheckLevelExpert();
  void UnCheckLevelCustom();

  void cmTable();

  DECLARE_RESPONSE_TABLE;

  StaticText *fast[Entries],
	     *status,
	     *slow[Entries];
  //RadioButtonContainer *game, *level;
  CheckBox *gameclear, *levelbeginner,
	   *gamecross, *levelintermediate,
	   *automark,  *levelexpert,
	   *autoopen,  *levelcustom,
	   *deviation;

  TextButton *ok;

  CheckBox *lastlevel;

  Layout CustomSave;
};


// This is the dialog box to ask for the player's name.
class EnterNameDlg : public Window
{
public:
  EnterNameDlg(const char* message);
  ~EnterNameDlg();

  char* Run();

protected:
  void PaintWindow(int x1, int y1, int x2, int y2);

  void KeyEnter(int) { ws.StopRunningEvents(); }
  void cmOK()	     { ws.StopRunningEvents(); }

  DECLARE_RESPONSE_TABLE;

  StaticText* msg;
  EditText*   name;
  TextButton* ok;

  static char lastname[NameLen];
};

#endif
